@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
@@ -17,11 +17,9 @@ const HOT_CORNER_ACTIVATION_TIMEOUT = 0.5;
 const STARTUP_ANIMATION_TIME = 0.2;
 const KEYBOARD_ANIMATION_TIME = 0.5;
 
-function LayoutManager() {
-    this._init.apply(this, arguments);
-}
+const LayoutManager = new Lang.Class({
+    Name: 'LayoutManager',
 
-LayoutManager.prototype = {
     _init: function () {
         this._rtl = (St.Widget.get_default_direction() == St.TextDirection.RTL);
         this.monitors = [];
@@ -45,7 +43,9 @@ LayoutManager.prototype = {
         this.trayBox.connect('allocation-changed',
                              Lang.bind(this, this._updateTrayBarrier));
 
-        this.keyboardBox = new St.BoxLayout({ name: 'keyboardBox' });
+        this.keyboardBox = new St.BoxLayout({ name: 'keyboardBox',
+                                              reactive: true,
+                                              track_hover: true });
         this.addChrome(this.keyboardBox, { visibleInFullscreen: true });
         this._keyboardHeightNotifyId = 0;
 
@@ -214,10 +214,10 @@ LayoutManager.prototype = {
         let monitorLeft = monitor.x, monitorRight = monitor.x + monitor.width;
         let primaryLeft = primary.x, primaryRight = primary.x + primary.width;
 
-        if ((monitorLeft >= primaryLeft && monitorLeft <= primaryRight) ||
-            (monitorRight >= primaryLeft && monitorRight <= primaryRight) ||
-            (primaryLeft >= monitorLeft && primaryLeft <= monitorRight) ||
-            (primaryRight >= monitorLeft && primaryRight <= monitorRight))
+        if ((monitorLeft >= primaryLeft && monitorLeft < primaryRight) ||
+            (monitorRight > primaryLeft && monitorRight <= primaryRight) ||
+            (primaryLeft >= monitorLeft && primaryLeft < monitorRight) ||
+            (primaryRight > monitorLeft && primaryRight <= monitorRight))
             return true;
 
         return false;
@@ -367,8 +367,12 @@ LayoutManager.prototype = {
     // Removes @actor from the chrome
     removeChrome: function(actor) {
         this._chrome.removeActor(actor);
+    },
+
+    findMonitorForActor: function(actor) {
+        return this._chrome.findMonitorForActor(actor);
     }
-};
+});
 Signals.addSignalMethods(LayoutManager.prototype);
 
 
@@ -376,11 +380,9 @@ Signals.addSignalMethods(LayoutManager.prototype);
 //
 // This class manages a "hot corner" that can toggle switching to
 // overview.
-function HotCorner() {
-    this._init();
-}
+const HotCorner = new Lang.Class({
+    Name: 'HotCorner',
 
-HotCorner.prototype = {
     _init : function() {
         // We use this flag to mark the case where the user has entered the
         // hot corner and has not left both the hot corner and a surrounding
@@ -431,9 +433,9 @@ HotCorner.prototype = {
                              Lang.bind(this, this._onCornerLeft));
 
         // Cache the three ripples instead of dynamically creating and destroying them.
-        this._ripple1 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0 });
-        this._ripple2 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0 });
-        this._ripple3 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0 });
+        this._ripple1 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
+        this._ripple2 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
+        this._ripple3 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
 
         Main.uiGroup.add_actor(this._ripple1);
         Main.uiGroup.add_actor(this._ripple2);
@@ -542,7 +544,7 @@ HotCorner.prototype = {
             return true;
         return false;
     }
-};
+});
 
 
 // This manages the shell "chrome"; the UI that's visible in the
@@ -555,11 +557,9 @@ const defaultParams = {
     affectsInputRegion: true
 };
 
-function Chrome() {
-    this._init.apply(this, arguments);
-}
+const Chrome = new Lang.Class({
+    Name: 'Chrome',
 
-Chrome.prototype = {
     _init: function(layoutManager) {
         this._layoutManager = layoutManager;
 
@@ -581,12 +581,13 @@ Chrome.prototype = {
 
         this._screenSaverActive = false;
         this._screenSaverProxy = new ScreenSaver.ScreenSaverProxy();
-        this._screenSaverProxy.connect('ActiveChanged', Lang.bind(this, this._onScreenSaverActiveChanged));
-        this._screenSaverProxy.GetActiveRemote(Lang.bind(this,
-            function(result, err) {
-                if (!err)
-                    this._onScreenSaverActiveChanged(this._screenSaverProxy, result);
-            }));
+        this._screenSaverProxy.connectSignal('ActiveChanged', Lang.bind(this, function(proxy, senderName, [isActive]) {
+            this._onScreenSaverActiveChanged(isActive);
+        }));
+        this._screenSaverProxy.GetActiveRemote(Lang.bind(this, function(result, err) {
+            if (!err)
+                this._onScreenSaverActiveChanged(result[0]);
+        }));
 
         this._relayout();
     },
@@ -619,7 +620,7 @@ Chrome.prototype = {
         // We can't use Params.parse here because we want to drop
         // the extra values like ancestorData.actor
         for (let prop in defaultParams) {
-            if (!params[prop])
+            if (!params.hasOwnProperty(prop))
                 params[prop] = ancestorData[prop];
         }
 
@@ -698,7 +699,7 @@ Chrome.prototype = {
             else if (this._inOverview)
                 visible = true;
             else if (!actorData.visibleInFullscreen &&
-                     this._findMonitorForActor(actorData.actor).inFullscreen)
+                     this.findMonitorForActor(actorData.actor).inFullscreen)
                 visible = false;
             else
                 visible = true;
@@ -727,7 +728,7 @@ Chrome.prototype = {
         this._queueUpdateRegions();
     },
 
-    _onScreenSaverActiveChanged: function(proxy, screenSaverActive) {
+    _onScreenSaverActiveChanged: function(screenSaverActive) {
         this._screenSaverActive = screenSaverActive;
         this._updateVisibility();
         this._queueUpdateRegions();
@@ -760,7 +761,7 @@ Chrome.prototype = {
 
     // This call guarantees that we return some monitor to simplify usage of it
     // In practice all tracked actors should be visible on some monitor anyway
-    _findMonitorForActor: function(actor) {
+    findMonitorForActor: function(actor) {
         let [x, y] = actor.get_transformed_position();
         let [w, h] = actor.get_transformed_size();
         let monitor = this._findMonitorForRect(x, y, w, h);
@@ -819,6 +820,18 @@ Chrome.prototype = {
                     monitor.inFullscreen = true;
             }
             if (layer == Meta.StackLayer.OVERRIDE_REDIRECT) {
+                // Check whether the window is screen sized
+                let isScreenSized =
+                    (window.x == 0 && window.y == 0 &&
+                    window.width == global.screen_width &&
+                    window.height == global.screen_height);
+
+                if (isScreenSized) {
+                    for (let i = 0; i < this._monitors.length; i++)
+                        this._monitors[i].inFullscreen = true;
+                }
+
+                // Or whether it is monitor sized
                 let monitor = this._findMonitorForWindow(window);
                 if (monitor &&
                     window.x <= monitor.x &&
@@ -962,4 +975,4 @@ Chrome.prototype = {
 
         return false;
     }
-};
+});
