@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Gdk = imports.gi.Gdk;
@@ -29,11 +29,9 @@ const State = {
     FADED_OUT: 4
 };
 
-function ModalDialog() {
-    this._init();
-}
+const ModalDialog = new Lang.Class({
+    Name: 'ModalDialog',
 
-ModalDialog.prototype = {
     _init: function(params) {
         params = Params.parse(params, { shellReactive: false,
                                         styleClass: null });
@@ -110,7 +108,7 @@ ModalDialog.prototype = {
         this._buttonLayout.destroy_children();
         this._actionKeys = {};
 
-        for (let i = 0; i < buttons.length; i ++) {
+        for (let i = 0; i < buttons.length; i++) {
             let buttonInfo = buttons[i];
             let label = buttonInfo['label'];
             let action = buttonInfo['action'];
@@ -303,5 +301,5 @@ ModalDialog.prototype = {
                                })
                          });
     }
-};
+});
 Signals.addSignalMethods(ModalDialog.prototype);
