@@ -1,5 +1,5 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*-
- *
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
+/*
  * Copyright 2011 Giovanni Campagna <scampa.giovanni@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,16 +30,14 @@ const St = imports.gi.St;
 
 const ModalDialog = imports.ui.modalDialog;
 const PopupMenu = imports.ui.popupMenu;
+const ShellEntry = imports.ui.shellEntry;
 
-function NetworkSecretDialog() {
-    this._init.apply(this, arguments);
-}
-
-NetworkSecretDialog.prototype = {
-    __proto__: ModalDialog.ModalDialog.prototype,
+const NetworkSecretDialog = new Lang.Class({
+    Name: 'NetworkSecretDialog',
+    Extends: ModalDialog.ModalDialog,
 
     _init: function(agent, requestId, connection, settingName, hints) {
-        ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: 'polkit-dialog' });
+        this.parent({ styleClass: 'polkit-dialog' });
 
         this._agent = agent;
         this._requestId = requestId;
@@ -91,6 +89,7 @@ NetworkSecretDialog.prototype = {
         }
 
         let secretTable = new St.Table({ style_class: 'network-dialog-secret-table' });
+        let initialFocusSet = false;
         let pos = 0;
         for (let i = 0; i < this._content.secrets.length; i++) {
             let secret = this._content.secrets[i];
@@ -102,6 +101,8 @@ NetworkSecretDialog.prototype = {
             secret.entry = new St.Entry({ style_class: 'polkit-dialog-password-entry',
                                           text: secret.value, can_focus: reactive,
                                           reactive: reactive });
+            ShellEntry.addContextMenu(secret.entry,
+                                      { isPassword: secret.password });
 
             if (secret.validate)
                 secret.valid = secret.validate(secret);
@@ -109,6 +110,12 @@ NetworkSecretDialog.prototype = {
                 secret.valid = secret.value.length > 0;
 
             if (reactive) {
+                if (!initialFocusSet) {
+                    this.setInitialKeyFocus(secret.entry);
+                    initialFocusSet = true;
+                }
+
+                secret.entry.clutter_text.connect('activate', Lang.bind(this, this._onOk));
                 secret.entry.clutter_text.connect('text-changed', Lang.bind(this, function() {
                     secret.value = secret.entry.get_text();
                     if (secret.validate)
@@ -120,34 +127,14 @@ NetworkSecretDialog.prototype = {
             } else
                 secret.valid = true;
 
-            secretTable.add(label, { row: pos, col: 0, x_align: St.Align.START, y_align: St.Align.START });
+            secretTable.add(label, { row: pos, col: 0, x_expand: false, x_fill: true, x_align: St.Align.START, y_align: St.Align.START });
             secretTable.add(secret.entry, { row: pos, col: 1, x_expand: true, x_fill: true, y_align: St.Align.END });
             pos++;
 
-            if (secret.password) {
+            if (secret.password)
                 secret.entry.clutter_text.set_password_char('\u25cf');
-
-                // FIXME: need a real checkbox here
-                let button = new St.Button({ button_mask: St.ButtonMask.ONE,
-                                             can_focus: true });
-                let checkbox = new St.BoxLayout({ vertical: false,
-                                                  style_class: 'network-dialog-show-password-checkbox' 
-                                                });
-                let _switch = new PopupMenu.Switch(false);
-                checkbox.add(_switch.actor);
-                checkbox.add(new St.Label({ text: _("Show password") }), { expand: true });
-                button.connect('clicked', function() {
-                    _switch.toggle();
-                    if (_switch.state)
-                        secret.entry.clutter_text.set_password_char('');
-                    else
-                        secret.entry.clutter_text.set_password_char('\u25cf');
-                });
-                button.child = checkbox;
-                secretTable.add(button, { row: pos, col: 1, x_expand: true, x_fill: true, y_fill: true })
-                pos++;
-            }
         }
+
         messageBox.add(secretTable);
 
         this._okButton = { label:  _("Connect"),
@@ -368,13 +355,11 @@ NetworkSecretDialog.prototype = {
 
         return content;
     }
-};
+});
 
-function NetworkAgent() {
-    this._init.apply(this, arguments);
-}
+const NetworkAgent = new Lang.Class({
+    Name: 'NetworkAgent',
 
-NetworkAgent.prototype = {
     _init: function() {
         this._native = new Shell.NetworkAgent({ auto_register: true,
                                                 identifier: 'org.gnome.Shell.NetworkAgent' });
@@ -397,4 +382,4 @@ NetworkAgent.prototype = {
         this._dialogs[requestId].close(global.get_current_time());
         this._dialogs[requestId].destroy();
     }
-};
\ No newline at end of file
+});
