@@ -23,6 +23,7 @@ const Params = imports.misc.params;
 const PlaceDisplay = imports.ui.placeDisplay;
 const Tweener = imports.ui.tweener;
 const ViewSelector = imports.ui.viewSelector;
+const Wanda = imports.ui.wanda;
 const WorkspacesView = imports.ui.workspacesView;
 const WorkspaceThumbnail = imports.ui.workspaceThumbnail;
 
@@ -46,11 +47,9 @@ const SwipeScrollResult = {
     CLICK: 2
 };
 
-function ShellInfo() {
-    this._init();
-}
+const ShellInfo = new Lang.Class({
+    Name: 'ShellInfo',
 
-ShellInfo.prototype = {
     _init: function() {
         this._source = null;
         this._undoCallback = null;
@@ -95,13 +94,11 @@ ShellInfo.prototype = {
 
         this._source.notify(notification);
     }
-};
+});
 
-function Overview() {
-    this._init.apply(this, arguments);
-}
+const Overview = new Lang.Class({
+    Name: 'Overview',
 
-Overview.prototype = {
     _init : function(params) {
         params = Params.parse(params, { isDummy: false });
 
@@ -112,7 +109,6 @@ Overview.prototype = {
         if (this.isDummy) {
             this.animationInProgress = false;
             this.visible = false;
-            this.workspaces = null;
             return;
         }
 
@@ -184,8 +180,6 @@ Overview.prototype = {
         this._lastActiveWorkspaceIndex = -1;
         this._lastHoveredWindow = null;
         this._needsFakePointerEvent = false;
-
-        this.workspaces = null;
     },
 
     // The members we construct that are implemented in JS might
@@ -208,6 +202,8 @@ Overview.prototype = {
         this._viewSelector.addViewTab('applications', _("Applications"), appView.actor, 'system-run');
 
         // Default search providers
+        // Wanda comes obviously first
+        this.addSearchProvider(new Wanda.WandaSearchProvider());
         this.addSearchProvider(new AppDisplay.AppSearchProvider());
         this.addSearchProvider(new AppDisplay.SettingsSearchProvider());
         this.addSearchProvider(new PlaceDisplay.PlaceSearchProvider());
@@ -591,13 +587,10 @@ Overview.prototype = {
 
         this._workspacesDisplay.show();
 
-        this.workspaces = this._workspacesDisplay.workspacesView;
-        global.overlay_group.add_actor(this.workspaces.actor);
-
         if (!this._desktopFade.child)
             this._desktopFade.child = this._getDesktopClone();
 
-        if (!this.workspaces.getActiveWorkspace().hasMaximizedWindows()) {
+        if (!this._workspacesDisplay.activeWorkspaceHasMaximizedWindows()) {
             this._desktopFade.opacity = 255;
             this._desktopFade.show();
             Tweener.addTween(this._desktopFade,
@@ -732,7 +725,7 @@ Overview.prototype = {
         this.animationInProgress = true;
         this._hideInProgress = true;
 
-        if (!this.workspaces.getActiveWorkspace().hasMaximizedWindows()) {
+        if (!this._workspacesDisplay.activeWorkspaceHasMaximizedWindows()) {
             this._desktopFade.opacity = 0;
             this._desktopFade.show();
             Tweener.addTween(this._desktopFade,
@@ -741,7 +734,7 @@ Overview.prototype = {
                                transition: 'easeOutQuad' });
         }
 
-        this.workspaces.hide();
+        this._workspacesDisplay.zoomFromOverview();
 
         // Make other elements fade out.
         Tweener.addTween(this._group,
@@ -783,9 +776,6 @@ Overview.prototype = {
 
         global.window_group.show();
 
-        this.workspaces.destroy();
-        this.workspaces = null;
-
         this._workspacesDisplay.hide();
 
         this._desktopFade.hide();
@@ -811,5 +801,5 @@ Overview.prototype = {
             this._needsFakePointerEvent = false;
         }
     }
-};
+});
 Signals.addSignalMethods(Overview.prototype);
