@@ -237,7 +237,7 @@ const AppMenuButton = new Lang.Class({
     Extends: PanelMenu.Button,
 
     _init: function(menuManager) {
-        this.parent(0.0, true);
+        this.parent(0.0, null, true);
 
         this._startingApps = [];
 
@@ -249,6 +249,7 @@ const AppMenuButton = new Lang.Class({
         let bin = new St.Bin({ name: 'appMenu' });
         this.actor.add_actor(bin);
 
+        this.actor.bind_property("reactive", this.actor, "can-focus", 0);
         this.actor.reactive = false;
         this._targetIsCurrent = false;
 
@@ -497,6 +498,9 @@ const AppMenuButton = new Lang.Class({
             return;
         }
 
+        if (!targetApp.is_on_workspace(workspace))
+            return;
+
         if (!this._targetIsCurrent) {
             this.actor.reactive = true;
             this._targetIsCurrent = true;
@@ -537,6 +541,7 @@ const AppMenuButton = new Lang.Class({
         let icon = targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
 
         this._label.setText(targetApp.get_name());
+        this.setName(targetApp.get_name());
 
         this._iconBox.set_child(icon);
         this._iconBox.show();
