@@ -389,6 +389,7 @@ const PopupMenuItem = new Lang.Class({
 
         this.label = new St.Label({ text: text });
         this.addActor(this.label);
+        this.actor.label_actor = this.label
     }
 });
 
@@ -773,6 +774,12 @@ const PopupSwitchMenuItem = new Lang.Class({
             this.toggle();
         }
 
+        // we allow pressing space to toggle the switch
+        // without closing the menu
+        if (event.type() == Clutter.EventType.KEY_PRESS &&
+            event.get_key_symbol() == Clutter.KEY_space)
+            return;
+
         this.parent(event);
     },
 
@@ -869,6 +876,10 @@ const PopupMenuBase = new Lang.Class({
         return menuItem;
     },
 
+    isEmpty: function() {
+        return this.box.get_children().length == 0;
+    },
+
     isChildMenu: function(menu) {
         return this._childMenus.indexOf(menu) != -1;
     },
@@ -1194,6 +1205,9 @@ const PopupMenu = new Lang.Class({
         if (this.isOpen)
             return;
 
+        if (this.isEmpty())
+            return;
+
         this.isOpen = true;
 
         this._boxPointer.setPosition(this.sourceActor, this._arrowAlignment);
@@ -1285,6 +1299,9 @@ const PopupSubMenu = new Lang.Class({
         if (this.isOpen)
             return;
 
+        if (this.isEmpty())
+            return;
+
         this.isOpen = true;
 
         this.actor.show();
@@ -1428,6 +1445,7 @@ const PopupSubMenuMenuItem = new Lang.Class({
 
         this.label = new St.Label({ text: text });
         this.addActor(this.label);
+        this.actor.label_actor = this.label;
         this._triangle = new St.Label({ text: '\u25B8' });
         this.addActor(this._triangle, { align: St.Align.END });
 
@@ -1528,6 +1546,9 @@ const PopupComboMenu = new Lang.Class({
         if (this.isOpen)
             return;
 
+        if (this.isEmpty())
+            return;
+
         this.isOpen = true;
 
         let [sourceX, sourceY] = this.sourceActor.get_transformed_position();
