@@ -3,9 +3,12 @@
 const Lang = imports.lang;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const Shell = imports.gi.Shell;
 
 const Config = imports.misc.config;
 const ExtensionSystem = imports.ui.extensionSystem;
+const ExtensionUtils = imports.misc.extensionUtils;
+const Flashspot = imports.ui.flashspot;
 const Main = imports.ui.main;
 
 const GnomeShellIface = <interface name="org.gnome.Shell">
@@ -30,15 +33,18 @@ const GnomeShellIface = <interface name="org.gnome.Shell">
     <arg type="i" direction="in" name="y"/>
     <arg type="i" direction="in" name="width"/>
     <arg type="i" direction="in" name="height"/>
+    <arg type="b" direction="in" name="flash"/>
     <arg type="s" direction="in" name="filename"/>
     <arg type="b" direction="out" name="success"/>
 </method>
 <method name="ScreenshotWindow">
     <arg type="b" direction="in" name="include_frame"/>
+    <arg type="b" direction="in" name="flash"/>
     <arg type="s" direction="in" name="filename"/>
     <arg type="b" direction="out" name="success"/>
 </method>
 <method name="Screenshot">
+    <arg type="b" direction="in" name="flash"/>
     <arg type="s" direction="in" name="filename"/>
     <arg type="b" direction="out" name="success"/>
 </method>
@@ -56,6 +62,9 @@ const GnomeShellIface = <interface name="org.gnome.Shell">
     <arg type="s" direction="in" name="uuid"/>
     <arg type="b" direction="out" name="success"/>
 </method>
+<method name="LaunchExtensionPrefs">
+    <arg type="s" direction="in" name="uuid"/>
+</method>
 <property name="OverviewActive" type="b" access="readwrite" />
 <property name="ApiVersion" type="i" access="read" />
 <property name="ShellVersion" type="s" access="read" />
@@ -109,6 +118,16 @@ const GnomeShell = new Lang.Class({
         return [success, returnValue];
     },
 
+    _onScreenshotComplete: function(obj, result, area, flash, invocation) {
+        if (flash) {
+            let flashspot = new Flashspot.Flashspot(area);
+            flashspot.fire();
+        }
+
+        let retval = GLib.Variant.new('(b)', [result]);
+        invocation.return_value(retval);
+    },
+
     /**
      * ScreenshotArea:
      * @x: The X coordinate of the area
@@ -123,12 +142,10 @@ const GnomeShell = new Lang.Class({
      *
      */
     ScreenshotAreaAsync : function (params, invocation) {
-        let [x, y, width, height, filename, callback] = params;
-        global.screenshot_area (x, y, width, height, filename,
-            function (obj, result) {
-                let retval = GLib.Variant.new('(b)', [result]);
-                invocation.return_value(retval);
-            });
+        let [x, y, width, height, flash, filename, callback] = params;
+        global.screenshot_area (x, y, width, height, filename, 
+                                Lang.bind(this, this._onScreenshotComplete, 
+                                          flash, invocation));
     },
 
     /**
@@ -141,8 +158,11 @@ const GnomeShell = new Lang.Class({
      * indicating whether the operation was successful or not.
      *
      */
-    ScreenshotWindow : function (include_frame, filename) {
-        return global.screenshot_window (include_frame, filename);
+    ScreenshotWindowAsync : function (params, invocation) {
+        let [include_frame, flash, filename] = params;
+        global.screenshot_window (include_frame, filename, 
+                                  Lang.bind(this, this._onScreenshotComplete, 
+                                            flash, invocation));
     },
 
     /**
@@ -155,17 +175,15 @@ const GnomeShell = new Lang.Class({
      *
      */
     ScreenshotAsync : function (params, invocation) {
-        let [filename] = params;
-        global.screenshot(filename,
-            function (obj, result) {
-                let retval = GLib.Variant.new('(b)', [result]);
-                invocation.return_value(retval);
-            });
+        let [flash, filename] = params;
+        global.screenshot(filename, 
+                          Lang.bind(this, this._onScreenshotComplete, 
+                                    flash, invocation));
     },
 
     ListExtensions: function() {
         let out = {};
-        for (let uuid in ExtensionSystem.extensionMeta) {
+        for (let uuid in ExtensionUtils.extensions) {
             let dbusObj = this.GetExtensionInfo(uuid);
             out[uuid] = dbusObj;
         }
@@ -173,10 +191,23 @@ const GnomeShell = new Lang.Class({
     },
 
     GetExtensionInfo: function(uuid) {
-        let meta = ExtensionSystem.extensionMeta[uuid] || {};
+        let extension = ExtensionUtils.extensions[uuid];
+        if (!extension)
+            return {};
+
+        let obj = {};
+        Lang.copyProperties(extension.metadata, obj);
+
+        // Only serialize the properties that we actually need.
+        const serializedProperties = ["type", "state", "path", "error", "hasPrefs"];
+
+        serializedProperties.forEach(function(prop) {
+            obj[prop] = extension[prop];
+        });
+
         let out = {};
-        for (let key in meta) {
-            let val = meta[key];
+        for (let key in obj) {
+            let val = obj[key];
             let type;
             switch (typeof val) {
             case 'string':
@@ -185,16 +216,27 @@ const GnomeShell = new Lang.Class({
             case 'number':
                 type = 'd';
                 break;
+            case 'boolean':
+                type = 'b';
+                break;
             default:
                 continue;
             }
             out[key] = GLib.Variant.new(type, val);
         }
+
         return out;
     },
 
     GetExtensionErrors: function(uuid) {
-        return ExtensionSystem.errors[uuid] || [];
+        let extension = ExtensionUtils.extensions[uuid];
+        if (!extension)
+            return [];
+
+        if (!extension.errors)
+            return [];
+
+        return extension.errors;
     },
 
     EnableExtension: function(uuid) {
@@ -219,6 +261,13 @@ const GnomeShell = new Lang.Class({
         return ExtensionSystem.uninstallExtensionFromUUID(uuid);
     },
 
+    LaunchExtensionPrefs: function(uuid) {
+        let appSys = Shell.AppSystem.get_default();
+        let app = appSys.lookup_app('gnome-shell-extension-prefs.desktop');
+        app.launch(global.display.get_current_time_roundtrip(),
+                   ['extension:///' + uuid], -1, null);
+    },
+
     get OverviewActive() {
         return Main.overview.visible;
     },
