@@ -113,6 +113,7 @@ const NMNetworkMenuItem = new Lang.Class({
         }
 
         this._label = new St.Label({ text: title });
+        this.actor.label_actor = this._label;
         this.addActor(this._label);
         this._icons = new St.BoxLayout({ style_class: 'nm-menu-item-icons' });
         this.addActor(this._icons, { align: St.Align.END });
@@ -1537,7 +1538,7 @@ const NMApplet = new Lang.Class({
     Extends: PanelMenu.SystemStatusButton,
 
     _init: function() {
-        this.parent('network-error', null);
+        this.parent('network-error', null, _("Network"));
 
         this._client = NMClient.Client.new();
 
