@@ -3,6 +3,7 @@
 #include "config.h"
 
 #include "shell-app-system.h"
+#include "shell-app-usage.h"
 #include <string.h>
 
 #include <gio/gio.h>
@@ -14,6 +15,7 @@
 
 #include "shell-app-private.h"
 #include "shell-window-tracker-private.h"
+#include "shell-app-system-private.h"
 #include "shell-global.h"
 #include "shell-util.h"
 #include "st.h"
@@ -24,6 +26,7 @@
 static const char*const vendor_prefixes[] = { "gnome-",
                                               "fedora-",
                                               "mozilla-",
+                                              "debian-",
                                               NULL };
 
 enum {
@@ -32,6 +35,7 @@ enum {
 };
 
 enum {
+  APP_STATE_CHANGED,
   INSTALLED_CHANGED,
   LAST_SIGNAL
 };
@@ -41,12 +45,13 @@ static guint signals[LAST_SIGNAL] = { 0 };
 struct _ShellAppSystemPrivate {
   GMenuTree *apps_tree;
 
-  GHashTable *entry_to_app;
+  GHashTable *running_apps;
+  GHashTable *id_to_app;
 
   GSList *known_vendor_prefixes;
 
   GMenuTree *settings_tree;
-  GHashTable *setting_entry_to_app;
+  GHashTable *setting_id_to_app;
 };
 
 static void shell_app_system_finalize (GObject *object);
@@ -61,13 +66,19 @@ static void shell_app_system_class_init(ShellAppSystemClass *klass)
 
   gobject_class->finalize = shell_app_system_finalize;
 
+  signals[APP_STATE_CHANGED] = g_signal_new ("app-state-changed",
+                                             SHELL_TYPE_APP_SYSTEM,
+                                             G_SIGNAL_RUN_LAST,
+                                             0,
+                                             NULL, NULL, NULL,
+                                             G_TYPE_NONE, 1,
+                                             SHELL_TYPE_APP);
   signals[INSTALLED_CHANGED] =
     g_signal_new ("installed-changed",
 		  SHELL_TYPE_APP_SYSTEM,
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ShellAppSystemClass, installed_changed),
-		  NULL, NULL,
-		  g_cclosure_marshal_VOID__VOID,
+          NULL, NULL, NULL,
 		  G_TYPE_NONE, 0);
 
   g_type_class_add_private (gobject_class, sizeof (ShellAppSystemPrivate));
@@ -82,12 +93,13 @@ shell_app_system_init (ShellAppSystem *self)
                                                    SHELL_TYPE_APP_SYSTEM,
                                                    ShellAppSystemPrivate);
 
-  priv->entry_to_app = g_hash_table_new_full (NULL, NULL,
-                                              (GDestroyNotify)gmenu_tree_item_unref,
-                                              (GDestroyNotify)g_object_unref);
-  priv->setting_entry_to_app = g_hash_table_new_full (NULL, NULL,
-                                                      (GDestroyNotify)gmenu_tree_item_unref,
-                                                      (GDestroyNotify)g_object_unref);
+  priv->running_apps = g_hash_table_new_full (NULL, NULL, (GDestroyNotify) g_object_unref, NULL);
+  priv->id_to_app = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                           NULL,
+                                           (GDestroyNotify)g_object_unref);
+  priv->setting_id_to_app = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                   NULL,
+                                                   (GDestroyNotify)g_object_unref);
 
   /* For now, we want to pick up Evince, Nautilus, etc.  We'll
    * handle NODISPLAY semantics at a higher level or investigate them
@@ -112,8 +124,9 @@ shell_app_system_finalize (GObject *object)
   g_object_unref (priv->apps_tree);
   g_object_unref (priv->settings_tree);
 
-  g_hash_table_destroy (priv->entry_to_app);
-  g_hash_table_destroy (priv->setting_entry_to_app);
+  g_hash_table_destroy (priv->running_apps);
+  g_hash_table_destroy (priv->id_to_app);
+  g_hash_table_destroy (priv->setting_id_to_app);
 
   g_slist_foreach (priv->known_vendor_prefixes, (GFunc)g_free, NULL);
   g_slist_free (priv->known_vendor_prefixes);
@@ -229,35 +242,10 @@ get_prefix_for_entry (GMenuTreeEntry *entry)
 }
 
 static void
-load_app_entry (ShellAppSystem *self,
-                GMenuTreeEntry *entry)
+get_flattened_entries_recurse (GMenuTreeDirectory *dir,
+                               GHashTable         *entry_set)
 {
-  char *prefix;
-  ShellApp *app;
-
-  if (g_hash_table_lookup (self->priv->entry_to_app, entry))
-    return;
-
-  prefix = get_prefix_for_entry (entry);
-
-  if (prefix
-      && !g_slist_find_custom (self->priv->known_vendor_prefixes, prefix,
-                               (GCompareFunc)g_strcmp0))
-    self->priv->known_vendor_prefixes = g_slist_append (self->priv->known_vendor_prefixes,
-                                                        prefix);
-  else
-    g_free (prefix);
-
-  app = _shell_app_new (entry);
-
-  g_hash_table_insert (self->priv->entry_to_app, gmenu_tree_item_ref (entry), app);
-}
-
-static void
-gather_apps_recurse (ShellAppSystem     *self,
-                     GMenuTreeDirectory *root)
-{
-  GMenuTreeIter *iter = gmenu_tree_directory_iter (root);
+  GMenuTreeIter *iter = gmenu_tree_directory_iter (dir);
   GMenuTreeItemType next_type;
 
   while ((next_type = gmenu_tree_iter_next (iter)) != GMENU_TREE_ITEM_INVALID)
@@ -268,14 +256,18 @@ gather_apps_recurse (ShellAppSystem     *self,
         {
         case GMENU_TREE_ITEM_ENTRY:
           {
-            item = gmenu_tree_iter_get_entry (iter);
-            load_app_entry (self, (GMenuTreeEntry*)item);
+            GMenuTreeEntry *entry;
+            item = entry = gmenu_tree_iter_get_entry (iter);
+            /* Key is owned by entry */
+            g_hash_table_replace (entry_set,
+                                  (char*)gmenu_tree_entry_get_desktop_file_id (entry),
+                                  gmenu_tree_item_ref (entry));
           }
           break;
         case GMENU_TREE_ITEM_DIRECTORY:
           {
             item = gmenu_tree_iter_get_directory (iter);
-            gather_apps_recurse (self, (GMenuTreeDirectory*)item);
+            get_flattened_entries_recurse ((GMenuTreeDirectory*)item, entry_set);
           }
           break;
         default:
@@ -288,46 +280,24 @@ gather_apps_recurse (ShellAppSystem     *self,
   gmenu_tree_iter_unref (iter);
 }
 
-static void
-gather_settings_recurse (ShellAppSystem     *self,
-                         GMenuTreeDirectory *root)
+static GHashTable *
+get_flattened_entries_from_tree (GMenuTree *tree)
 {
-  GMenuTreeIter *iter = gmenu_tree_directory_iter (root);
-  GMenuTreeItemType next_type;
+  GHashTable *table;
+  GMenuTreeDirectory *root;
 
-  while ((next_type = gmenu_tree_iter_next (iter)) != GMENU_TREE_ITEM_INVALID)
-    {
-      gpointer item = NULL;
+  table = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                 (GDestroyNotify) NULL,
+                                 (GDestroyNotify) gmenu_tree_item_unref);
 
-      switch (next_type)
-        {
-        case GMENU_TREE_ITEM_ENTRY:
-          {
-            ShellApp *app;
+  root = gmenu_tree_get_root_directory (tree);
+  
+  if (root != NULL)
+    get_flattened_entries_recurse (root, table);
 
-            item = gmenu_tree_iter_get_entry (iter);
-            if (g_hash_table_lookup (self->priv->setting_entry_to_app, item))
-              return;
-            
-            app = _shell_app_new (item);
-
-            g_hash_table_insert (self->priv->setting_entry_to_app, gmenu_tree_item_ref (item), app);
-          }
-          break;
-        case GMENU_TREE_ITEM_DIRECTORY:
-          {
-            item = gmenu_tree_iter_get_directory (iter);
-            gather_settings_recurse (self, (GMenuTreeDirectory*)item);
-          }
-          break;
-        default:
-          break;
-        }
-      if (item != NULL)
-        gmenu_tree_item_unref (item);
-    }
-
-  gmenu_tree_iter_unref (iter);
+  gmenu_tree_item_unref (root);
+  
+  return table;
 }
 
 static void
@@ -336,11 +306,14 @@ on_apps_tree_changed_cb (GMenuTree *tree,
 {
   ShellAppSystem *self = SHELL_APP_SYSTEM (user_data);
   GError *error = NULL;
-  GMenuTreeDirectory *root;
+  GHashTable *new_apps;
+  GHashTableIter iter;
+  gpointer key, value;
+  GSList *removed_apps = NULL;
+  GSList *removed_node;
 
   g_assert (tree == self->priv->apps_tree);
 
-  g_hash_table_remove_all (self->priv->entry_to_app);
   g_slist_foreach (self->priv->known_vendor_prefixes, (GFunc)g_free, NULL);
   g_slist_free (self->priv->known_vendor_prefixes);
   self->priv->known_vendor_prefixes = NULL;
@@ -351,13 +324,74 @@ on_apps_tree_changed_cb (GMenuTree *tree,
       return;
     }
 
-  root = gmenu_tree_get_root_directory (self->priv->apps_tree);
-
-  if (root)
+  new_apps = get_flattened_entries_from_tree (self->priv->apps_tree);
+  g_hash_table_iter_init (&iter, new_apps);
+  while (g_hash_table_iter_next (&iter, &key, &value))
     {
-      gather_apps_recurse (self, root);
-      gmenu_tree_item_unref (root);
+      const char *id = key;
+      GMenuTreeEntry *entry = value;
+      GMenuTreeEntry *old_entry;
+      char *prefix;
+      ShellApp *app;
+      
+      prefix = get_prefix_for_entry (entry);
+      
+      if (prefix != NULL
+          && !g_slist_find_custom (self->priv->known_vendor_prefixes, prefix,
+                                   (GCompareFunc)g_strcmp0))
+        self->priv->known_vendor_prefixes = g_slist_append (self->priv->known_vendor_prefixes,
+                                                            prefix);
+      else
+        g_free (prefix);
+      
+      app = g_hash_table_lookup (self->priv->id_to_app, id);
+      if (app != NULL)
+        {
+          /* We hold a reference to the original entry temporarily,
+           * because otherwise the hash table would be referencing
+           * potentially free'd memory until we replace it below with
+           * the new data.
+           */
+          old_entry = shell_app_get_tree_entry (app);
+          gmenu_tree_item_ref (old_entry);
+          _shell_app_set_entry (app, entry);
+          g_object_ref (app);  /* Extra ref, removed in _replace below */
+        }
+      else
+        {
+          old_entry = NULL;
+          app = _shell_app_new (entry);
+        }
+      /* Note that "id" is owned by app->entry.  Since we're always
+       * setting a new entry, even if the app already exists in the
+       * hash table we need to replace the key so that the new id
+       * string is pointed to.
+       */
+      g_hash_table_replace (self->priv->id_to_app, (char*)id, app);
+
+      if (old_entry)
+        gmenu_tree_item_unref (old_entry);
     }
+  /* Now iterate over the apps again; we need to unreference any apps
+   * which have been removed.  The JS code may still be holding a
+   * reference; that's fine.
+   */
+  g_hash_table_iter_init (&iter, self->priv->id_to_app);
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      const char *id = key;
+      
+      if (!g_hash_table_lookup (new_apps, id))
+        removed_apps = g_slist_prepend (removed_apps, (char*)id);
+    }
+  for (removed_node = removed_apps; removed_node; removed_node = removed_node->next)
+    {
+      const char *id = removed_node->data;
+      g_hash_table_remove (self->priv->id_to_app, id);
+    }
+  g_slist_free (removed_apps);
+      
+  g_hash_table_destroy (new_apps);
 
   g_signal_emit (self, signals[INSTALLED_CHANGED], 0);
 }
@@ -368,24 +402,32 @@ on_settings_tree_changed_cb (GMenuTree *tree,
 {
   ShellAppSystem *self = SHELL_APP_SYSTEM (user_data);
   GError *error = NULL;
-  GMenuTreeDirectory *root;
+  GHashTable *new_settings;
+  GHashTableIter iter;
+  gpointer key, value;
 
   g_assert (tree == self->priv->settings_tree);
 
-  g_hash_table_remove_all (self->priv->setting_entry_to_app);
+  g_hash_table_remove_all (self->priv->setting_id_to_app);
   if (!gmenu_tree_load_sync (self->priv->settings_tree, &error))
     {
       g_warning ("Failed to load settings: %s", error->message);
       return;
     }
 
-  root = gmenu_tree_get_root_directory (self->priv->settings_tree);
+  new_settings = get_flattened_entries_from_tree (tree);
 
-  if (root)
+  g_hash_table_iter_init (&iter, new_settings);
+  while (g_hash_table_iter_next (&iter, &key, &value))
     {
-      gather_settings_recurse (self, root);
-      gmenu_tree_item_unref (root);
+      const char *id = key;
+      GMenuTreeEntry *entry = value;
+      ShellApp *app;
+
+      app = _shell_app_new (entry);
+      g_hash_table_replace (self->priv->setting_id_to_app, (char*)id, app);
     }
+  g_hash_table_destroy (new_settings);
 }
 
 /**
@@ -421,7 +463,6 @@ ShellApp *
 shell_app_system_lookup_setting (ShellAppSystem *self,
                                  const char     *id)
 {
-  GMenuTreeEntry *entry;
   ShellApp *app;
 
   /* Actually defer to the main app set if there's overlap */
@@ -429,18 +470,7 @@ shell_app_system_lookup_setting (ShellAppSystem *self,
   if (app != NULL)
     return app;
 
-  entry = gmenu_tree_get_entry_by_id (self->priv->settings_tree, id);
-  if (entry == NULL)
-    return NULL;
-  
-  app = g_hash_table_lookup (self->priv->setting_entry_to_app, entry);
-  if (app != NULL)
-    return app;
-  
-  app = _shell_app_new (entry);
-  g_hash_table_insert (self->priv->setting_entry_to_app, gmenu_tree_item_ref (entry), app); 
-
-  return app;
+  return g_hash_table_lookup (self->priv->setting_id_to_app, id);
 }
 
 /**
@@ -470,13 +500,7 @@ ShellApp *
 shell_app_system_lookup_app (ShellAppSystem   *self,
                              const char       *id)
 {
-  GMenuTreeEntry *entry;
-
-  entry = gmenu_tree_get_entry_by_id (self->priv->apps_tree, id);
-  if (entry == NULL)
-    return NULL;
-
-  return g_hash_table_lookup (self->priv->entry_to_app, entry);
+  return g_hash_table_lookup (self->priv->id_to_app, id);
 }
 
 /**
@@ -492,7 +516,10 @@ ShellApp *
 shell_app_system_lookup_app_by_tree_entry (ShellAppSystem  *self,
                                            GMenuTreeEntry  *entry)
 {
-  return g_hash_table_lookup (self->priv->entry_to_app, entry);
+  /* If we looked up directly in ->entry_to_app, we'd lose the
+   * override of running apps.  Thus, indirect through the id.
+   */
+  return shell_app_system_lookup_app (self, gmenu_tree_entry_get_desktop_file_id (entry));
 }
 
 /**
@@ -578,47 +605,95 @@ shell_app_system_get_all (ShellAppSystem  *self)
   GHashTableIter iter;
   gpointer key, value;
 
-  g_hash_table_iter_init (&iter, self->priv->entry_to_app);
+  g_hash_table_iter_init (&iter, self->priv->id_to_app);
   while (g_hash_table_iter_next (&iter, &key, &value))
     {
       ShellApp *app = value;
+      
       if (!g_desktop_app_info_get_nodisplay (shell_app_get_app_info (app)))
         result = g_slist_prepend (result, app);
     }
   return result;
 }
 
-static gint
-compare_apps_by_name (gconstpointer a,
-                      gconstpointer b,
-                      gpointer      data)
+void
+_shell_app_system_notify_app_state_changed (ShellAppSystem *self,
+                                            ShellApp       *app)
 {
+  ShellAppState state = shell_app_get_state (app);
+
+  switch (state)
+    {
+    case SHELL_APP_STATE_RUNNING:
+      g_hash_table_insert (self->priv->running_apps, g_object_ref (app), NULL);
+      break;
+    case SHELL_APP_STATE_STARTING:
+      break;
+    case SHELL_APP_STATE_STOPPED:
+      g_hash_table_remove (self->priv->running_apps, app);
+      break;
+    }
+  g_signal_emit (self, signals[APP_STATE_CHANGED], 0, app);
+}
+
+/**
+ * shell_app_system_get_running:
+ * @self: A #ShellAppSystem
+ *
+ * Returns the set of applications which currently have at least one
+ * open window in the given context.  The returned list will be sorted
+ * by shell_app_compare().
+ *
+ * Returns: (element-type ShellApp) (transfer container): Active applications
+ */
+GSList *
+shell_app_system_get_running (ShellAppSystem *self)
+{
+  gpointer key, value;
+  GSList *ret;
+  GHashTableIter iter;
+
+  g_hash_table_iter_init (&iter, self->priv->running_apps);
+
+  ret = NULL;
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      ShellApp *app = key;
+
+      ret = g_slist_prepend (ret, app);
+    }
+
+  ret = g_slist_sort (ret, (GCompareFunc)shell_app_compare);
+
+  return ret;
+}
+
+
+static gint
+compare_apps_by_usage (gconstpointer a,
+                       gconstpointer b,
+                       gpointer      data)
+{
+  ShellAppUsage *usage = shell_app_usage_get_default ();
+
   ShellApp *app_a = (ShellApp*)a;
   ShellApp *app_b = (ShellApp*)b;
 
-  return shell_app_compare_by_name (app_a, app_b);
+  return shell_app_usage_compare (usage, "", app_a, app_b);
 }
 
 static GSList *
 sort_and_concat_results (ShellAppSystem *system,
-                         GSList         *multiple_prefix_matches,
                          GSList         *prefix_matches,
-                         GSList         *multiple_substring_matches,
                          GSList         *substring_matches)
 {
-  multiple_prefix_matches = g_slist_sort_with_data (multiple_prefix_matches,
-                                                    compare_apps_by_name,
-                                                    system);
   prefix_matches = g_slist_sort_with_data (prefix_matches,
-                                           compare_apps_by_name,
+                                           compare_apps_by_usage,
                                            system);
-  multiple_substring_matches = g_slist_sort_with_data (multiple_substring_matches,
-                                                       compare_apps_by_name,
-                                                       system);
   substring_matches = g_slist_sort_with_data (substring_matches,
-                                              compare_apps_by_name,
+                                              compare_apps_by_usage,
                                               system);
-  return g_slist_concat (multiple_prefix_matches, g_slist_concat (prefix_matches, g_slist_concat (multiple_substring_matches, substring_matches)));
+  return g_slist_concat (prefix_matches, substring_matches);
 }
 
 /**
@@ -645,9 +720,7 @@ search_tree (ShellAppSystem *self,
              GSList         *terms,
              GHashTable     *apps)
 {
-  GSList *multiple_prefix_results = NULL;
   GSList *prefix_results = NULL;
-  GSList *multiple_subtring_results = NULL;
   GSList *substring_results = NULL;
   GSList *normalized_terms;
   GHashTableIter iter;
@@ -662,14 +735,13 @@ search_tree (ShellAppSystem *self,
       ShellApp *app = value;
       (void)id;
       _shell_app_do_match (app, normalized_terms,
-                           &multiple_prefix_results, &prefix_results,
-                           &multiple_subtring_results, &substring_results);
+                           &prefix_results,
+                           &substring_results);
     }
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
 
-  return sort_and_concat_results (self, multiple_prefix_results, prefix_results,
-                                  multiple_subtring_results, substring_results);
+  return sort_and_concat_results (self, prefix_results, substring_results);
 
 }
 
@@ -686,7 +758,7 @@ GSList *
 shell_app_system_initial_search (ShellAppSystem  *self,
                                  GSList          *terms)
 {
-  return search_tree (self, terms, self->priv->entry_to_app);
+  return search_tree (self, terms, self->priv->id_to_app);
 }
 
 /**
@@ -708,9 +780,7 @@ shell_app_system_subsearch (ShellAppSystem   *system,
                             GSList           *terms)
 {
   GSList *iter;
-  GSList *multiple_prefix_results = NULL;
   GSList *prefix_results = NULL;
-  GSList *multiple_substring_results = NULL;
   GSList *substring_results = NULL;
   GSList *normalized_terms = normalize_terms (terms);
 
@@ -719,8 +789,8 @@ shell_app_system_subsearch (ShellAppSystem   *system,
       ShellApp *app = iter->data;
       
       _shell_app_do_match (app, normalized_terms,
-                           &multiple_prefix_results, &prefix_results,
-                           &multiple_substring_results, &substring_results);
+                           &prefix_results,
+                           &substring_results);
     }
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
@@ -728,7 +798,7 @@ shell_app_system_subsearch (ShellAppSystem   *system,
   /* Note that a shorter term might have matched as a prefix, but
      when extended only as a substring, so we have to redo the
      sort rather than reusing the existing ordering */
-  return sort_and_concat_results (system, multiple_prefix_results, prefix_results, multiple_substring_results, substring_results);
+  return sort_and_concat_results (system, prefix_results, substring_results);
 }
 
 /**
@@ -744,5 +814,5 @@ GSList *
 shell_app_system_search_settings (ShellAppSystem  *self,
                                   GSList          *terms)
 {
-  return search_tree (self, terms, self->priv->setting_entry_to_app);
+  return search_tree (self, terms, self->priv->setting_id_to_app);
 }
