@@ -25,6 +25,7 @@
 #include <girepository.h>
 #include <meta/display.h>
 #include <meta/util.h>
+#include <meta/meta-shaped-texture.h>
 
 /* Memory report bits */
 #ifdef HAVE_MALLINFO
@@ -34,8 +35,8 @@
 #include "shell-enum-types.h"
 #include "shell-global-private.h"
 #include "shell-jsapi-compat-private.h"
-#include "shell-marshal.h"
 #include "shell-perf-log.h"
+#include "shell-screen-grabber.h"
 #include "shell-window-tracker.h"
 #include "shell-wm.h"
 #include "st.h"
@@ -311,8 +312,7 @@ shell_global_class_init (ShellGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    _shell_marshal_VOID__INT_INT,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
   /* Emitted from gnome-shell-plugin.c during event handling */
@@ -321,8 +321,7 @@ shell_global_class_init (ShellGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 0);
 
   /* Emitted from gnome-shell-plugin.c during event handling */
@@ -331,8 +330,7 @@ shell_global_class_init (ShellGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 0);
 
   shell_global_signals[NOTIFY_ERROR] =
@@ -340,8 +338,7 @@ shell_global_class_init (ShellGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    gi_cclosure_marshal_generic,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 2,
                     G_TYPE_STRING,
                     G_TYPE_STRING);
@@ -1180,70 +1177,6 @@ shell_global_destroy_pointer_barrier (ShellGlobal *global, guint32 barrier)
 #endif
 }
 
-
-/**
- * shell_global_add_extension_importer:
- * @target_object_script: JavaScript code evaluating to a target object
- * @target_property: Name of property to use for importer
- * @directory: Source directory:
- * @error: A #GError
- *
- * This function sets a property named @target_property on the object
- * resulting from the evaluation of @target_object_script code, which
- * acts as a GJS importer for directory @directory.
- *
- * Returns: %TRUE on success
- */
-gboolean
-shell_global_add_extension_importer (ShellGlobal *global,
-                                     const char  *target_object_script,
-                                     const char  *target_property,
-                                     const char  *directory,
-                                     GError     **error)
-{
-  jsval target_object;
-  JSContext *context = gjs_context_get_native_context (global->js_context);
-  char *search_path[2] = { 0, 0 };
-
-  JS_BeginRequest (context);
-
-  // This is a bit of a hack; ideally we'd be able to pass our target
-  // object directly into this function, but introspection doesn't
-  // support that at the moment.  Instead evaluate a string to get it.
-  if (!JS_EvaluateScript(context,
-                         JS_GetGlobalObject(context),
-                         target_object_script,
-                         strlen (target_object_script),
-                         "<target_object_script>",
-                         0,
-                         &target_object))
-    {
-      char *message;
-      gjs_log_exception(context,
-                        &message);
-      g_set_error(error,
-                  G_IO_ERROR,
-                  G_IO_ERROR_FAILED,
-                  "%s", message ? message : "(unknown)");
-      g_free(message);
-      goto out_error;
-    }
-
-  if (!JSVAL_IS_OBJECT (target_object))
-    {
-      g_error ("shell_global_add_extension_importer: invalid target object");
-      goto out_error;
-    }
-
-  search_path[0] = (char*)directory;
-  gjs_define_importer (context, JSVAL_TO_OBJECT (target_object), target_property, (const char **)search_path, FALSE);
-  JS_EndRequest (context);
-  return TRUE;
- out_error:
-  JS_EndRequest (context);
-  return FALSE;
-}
-
 /* Code to close all file descriptors before we exec; copied from gspawn.c in GLib.
  *
  * Authors: Padraig O'Briain, Matthias Clasen, Lennart Poettering
@@ -1965,25 +1898,67 @@ shell_global_launch_calendar_server (ShellGlobal *global)
   g_free (calendar_server_exe);
 }
 
+static void
+on_screenshot_written (GObject *source,
+                       GAsyncResult *result,
+                       gpointer user_data)
+{
+  _screenshot_data *screenshot_data = (_screenshot_data*) user_data;
+  if (screenshot_data->callback)
+    screenshot_data->callback (screenshot_data->global, 
+                               g_simple_async_result_get_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (result)),
+                               &screenshot_data->screenshot_area);
+
+  cairo_surface_destroy (screenshot_data->image);
+  g_free (screenshot_data->filename);
+  g_free (screenshot_data);
+}
+
+static void
+write_screenshot_thread (GSimpleAsyncResult *result,
+                         GObject *object,
+                         GCancellable *cancellable)
+{
+  cairo_status_t status;
+  _screenshot_data *screenshot_data = g_async_result_get_user_data (G_ASYNC_RESULT (result));
+  g_assert (screenshot_data != NULL);
+
+  status = cairo_surface_write_to_png (screenshot_data->image, screenshot_data->filename);
+  g_simple_async_result_set_op_res_gboolean (result, status == CAIRO_STATUS_SUCCESS);
+}
+
+static void
+do_grab_screenshot (_screenshot_data *screenshot_data,
+                    int               x,
+                    int               y,
+                    int               width,
+                    int               height)
+{
+  ShellScreenGrabber *grabber;
+  static const cairo_user_data_key_t key;
+  guchar *data;
+
+  grabber = shell_screen_grabber_new ();
+  data = shell_screen_grabber_grab (grabber, x, y, width, height);
+  g_object_unref (grabber);
+
+  screenshot_data->image = cairo_image_surface_create_for_data (data, CAIRO_FORMAT_RGB24,
+                                                               width, height, width * 4);
+  cairo_surface_set_user_data (screenshot_data->image, &key,
+                               data, (cairo_destroy_func_t)g_free);
+}
+
 static void
 grab_screenshot (ClutterActor *stage,
                  _screenshot_data *screenshot_data)
 {
   MetaScreen *screen = shell_global_get_screen (screenshot_data->global);
-  cairo_status_t status;
-  cairo_surface_t *image;
-  guchar *data;
   int width, height;
+  GSimpleAsyncResult *result;
 
   meta_plugin_query_screen_size (screenshot_data->global->plugin, &width, &height);
-  image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
-  data = cairo_image_surface_get_data (image);
 
-  cogl_flush();
-
-  cogl_read_pixels (0, 0, width, height, COGL_READ_PIXELS_COLOR_BUFFER, CLUTTER_CAIRO_FORMAT_ARGB32, data);
-
-  cairo_surface_mark_dirty (image);
+  do_grab_screenshot (screenshot_data, 0, 0, width, height);
 
   if (meta_screen_get_n_monitors (screen) > 1)
     {
@@ -2009,7 +1984,7 @@ grab_screenshot (ClutterActor *stage,
       cairo_region_xor (stage_region, screen_region);
       cairo_region_destroy (screen_region);
 
-      cr = cairo_create (image);
+      cr = cairo_create (screenshot_data->image);
 
       for (i = 0; i < cairo_region_num_rectangles (stage_region); i++)
         {
@@ -2023,41 +1998,34 @@ grab_screenshot (ClutterActor *stage,
       cairo_region_destroy (stage_region);
     }
 
-
-  status = cairo_surface_write_to_png (image, screenshot_data->filename);
-  cairo_surface_destroy (image);
-
-  if (screenshot_data->callback)
-    screenshot_data->callback (screenshot_data->global, status == CAIRO_STATUS_SUCCESS);
+  screenshot_data->screenshot_area.x = 0;
+  screenshot_data->screenshot_area.y = 0;
+  screenshot_data->screenshot_area.width = width;
+  screenshot_data->screenshot_area.height = height;
 
   g_signal_handlers_disconnect_by_func (stage, (void *)grab_screenshot, (gpointer)screenshot_data);
-  g_free (screenshot_data->filename);
-  g_free (screenshot_data);
+
+  result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, grab_screenshot);
+  g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
+  g_object_unref (result);
 }
 
 static void
 grab_area_screenshot (ClutterActor *stage,
                       _screenshot_data *screenshot_data)
 {
-  cairo_status_t status;
-  cairo_surface_t *image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, screenshot_data->width, screenshot_data->height);
-  guchar *data = cairo_image_surface_get_data (image);
+  GSimpleAsyncResult *result;
 
-  cogl_flush();
-
-  cogl_read_pixels (screenshot_data->x, screenshot_data->y, screenshot_data->width, screenshot_data->height,
-                    COGL_READ_PIXELS_COLOR_BUFFER, CLUTTER_CAIRO_FORMAT_ARGB32, data);
-
-  cairo_surface_mark_dirty (image);
-  status = cairo_surface_write_to_png (image, screenshot_data->filename);
-  cairo_surface_destroy (image);
-
-  if (screenshot_data->callback)
-    screenshot_data->callback (screenshot_data->global, status == CAIRO_STATUS_SUCCESS);
+  do_grab_screenshot (screenshot_data,
+                      screenshot_data->screenshot_area.x,
+                      screenshot_data->screenshot_area.y,
+                      screenshot_data->screenshot_area.width,
+                      screenshot_data->screenshot_area.height);
 
   g_signal_handlers_disconnect_by_func (stage, (void *)grab_area_screenshot, (gpointer)screenshot_data);
-  g_free (screenshot_data->filename);
-  g_free (screenshot_data);
+  result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, grab_area_screenshot);
+  g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
+  g_object_unref (result);
 }
 
 /**
@@ -2073,8 +2041,8 @@ grab_area_screenshot (ClutterActor *stage,
  */
 void
 shell_global_screenshot (ShellGlobal  *global,
-                        const char *filename,
-                        ShellGlobalScreenshotCallback callback)
+                         const char *filename,
+                         ShellGlobalScreenshotCallback callback)
 {
   ClutterActor *stage;
   _screenshot_data *data = g_new0 (_screenshot_data, 1);
@@ -2119,10 +2087,10 @@ shell_global_screenshot_area (ShellGlobal  *global,
 
   data->global = global;
   data->filename = g_strdup (filename);
-  data->x = x;
-  data->y = y;
-  data->width = width;
-  data->height = height;
+  data->screenshot_area.x = x;
+  data->screenshot_area.y = y;
+  data->screenshot_area.width = width;
+  data->screenshot_area.height = height;
   data->callback = callback;
 
   stage = CLUTTER_ACTOR (meta_plugin_get_stage (global->plugin));
@@ -2138,60 +2106,69 @@ shell_global_screenshot_area (ShellGlobal  *global,
  * @include_frame: Whether to include the frame or not
  *
  * @filename: The filename for the screenshot
+ * @callback: (scope async): function to call returning success or failure
+ * of the async grabbing
  *
  * Takes a screenshot of the focused window (optionally omitting the frame)
  * in @filename as png image.
  *
- * Return value: success or failure.
  */
-gboolean
+void
 shell_global_screenshot_window (ShellGlobal  *global,
                                 gboolean include_frame,
-                                const char *filename)
+                                const char *filename,
+                                ShellGlobalScreenshotCallback callback)
 {
-  CoglHandle texture;
-  cairo_surface_t *image;
-  guchar *data;
+  GSimpleAsyncResult *result;
+
+  _screenshot_data *screenshot_data = g_new0 (_screenshot_data, 1);
 
   MetaScreen *screen = meta_plugin_get_screen (global->plugin);
   MetaDisplay *display = meta_screen_get_display (screen);
   MetaWindow *window = meta_display_get_focus_window (display);
   ClutterActor *window_actor;
+  gfloat actor_x, actor_y;
+  MetaShapedTexture *stex;
+  MetaRectangle rect;
+  cairo_rectangle_int_t clip;
 
-  cairo_status_t status;
+  screenshot_data->global = global;
+  screenshot_data->filename = g_strdup (filename);
+  screenshot_data->callback = callback;
 
   window_actor = CLUTTER_ACTOR (meta_window_get_compositor_private (window));
-  texture = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (meta_window_actor_get_texture (META_WINDOW_ACTOR (window_actor))));
+  clutter_actor_get_position (window_actor, &actor_x, &actor_y);
 
-  if (!include_frame)
+  if (include_frame || !meta_window_get_frame (window))
     {
-      MetaRectangle *window_rect = meta_window_get_rect (window);
-      texture = cogl_texture_new_from_sub_texture (texture,
-                                                   window_rect->x,
-                                                   window_rect->y,
-                                                   window_rect->width,
-                                                   window_rect->height);
+      meta_window_get_outer_rect (window, &rect);
 
-      image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                          window_rect->width,
-                                          window_rect->height);
+      screenshot_data->screenshot_area.x = rect.x;
+      screenshot_data->screenshot_area.y = rect.y;
+
+      clip.x = rect.x - (gint) actor_x;
+      clip.y = rect.y - (gint) actor_y;
     }
   else
-    image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                        clutter_actor_get_width (window_actor),
-                                        clutter_actor_get_height (window_actor));
+    {
+      rect = *meta_window_get_rect (window);
 
-  data = cairo_image_surface_get_data (image);
+      screenshot_data->screenshot_area.x = (gint) actor_x + rect.x;
+      screenshot_data->screenshot_area.y = (gint) actor_y + rect.y;
 
-  cogl_flush();
+      clip.x = rect.x;
+      clip.y = rect.y;
+    }
 
-  cogl_texture_get_data (texture, CLUTTER_CAIRO_FORMAT_ARGB32, 0, data);
+  clip.width = screenshot_data->screenshot_area.width = rect.width;
+  clip.height = screenshot_data->screenshot_area.height = rect.height;
 
-  cairo_surface_mark_dirty (image);
-  status = cairo_surface_write_to_png (image, filename);
-  cairo_surface_destroy (image);
+  stex = META_SHAPED_TEXTURE (meta_window_actor_get_texture (META_WINDOW_ACTOR (window_actor)));
+  screenshot_data->image = meta_shaped_texture_get_image (stex, &clip);
 
-  return status == CAIRO_STATUS_SUCCESS;
+  result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, shell_global_screenshot_window);
+  g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
+  g_object_unref (result);
 }
 
 /**
