@@ -35,7 +35,6 @@
 #include "st-widget.h"
 
 #include "st-label.h"
-#include "st-marshal.h"
 #include "st-private.h"
 #include "st-texture-cache.h"
 #include "st-theme-context.h"
@@ -995,8 +994,7 @@ st_widget_class_init (StWidgetClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (StWidgetClass, style_changed),
-                  NULL, NULL,
-                  _st_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -1011,8 +1009,7 @@ st_widget_class_init (StWidgetClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (StWidgetClass, popup_menu),
-                  NULL, NULL,
-                  _st_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
@@ -2441,7 +2438,7 @@ st_ui_root_destroyed (ClutterActor *actor,
 }
 
 /**
- * st_get_ui_root:
+ * st_set_ui_root:
  * @stage: a #ClutterStage
  * @container: (allow-none): the new UI root
  *
