@@ -29,7 +29,6 @@ int main (int argc, char **argv)
   ClutterAnimation *animation;
   ClutterColor red, green, blue;
 
-  g_thread_init (NULL);
   gst_init (&argc, &argv);
   if (clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
     return 1;
@@ -37,7 +36,8 @@ int main (int argc, char **argv)
   clutter_color_from_string (&red, "red");
   clutter_color_from_string (&green, "green");
   clutter_color_from_string (&blue, "blue");
-  stage = clutter_stage_get_default ();
+  stage = clutter_stage_new ();
+  g_signal_connect (stage, "destroy", G_CALLBACK (clutter_main_quit), NULL);
 
   text = g_object_new (CLUTTER_TYPE_TEXT,
 		       "text", "Red",
